function [cmap]=buildcmap(colors)
% [cmap]=buildcmap(colors)
%
% This function can be used to build your own custom colormaps. Imagine if
% you want to display rainfall distribution map. You want a colormap which
% ideally brings rainfall in mind, which is not achiveved by colormaps such
% as winter, cool or jet and such. A gradient of white to blue will do the
% task, but you might also use a more complex gradient (such as
% white+blue+red or colors='wbr'). This function can be use to build any
% colormap using main colors rgbcmyk. In image processing, w (white) can be
% used as the first color so that in the output, the background (usually
% with 0 values) appears white. In the example of rainfall map, 'wb' will
% produce a rainfall density map where the background (if its DN values are
% 0) will appear as white.
%
% Inputs:
%  colors: string (char) of color codes, any sequence of rgbcmywk
%  representing different colors (such as 'b' for blue) is acceptable. If a
%  gradient of white to blue is needed, colors would be 'wb'; a rainbow of
%  white+blue+red+green would be 'wbrg'.
%
% Example:
%  [cmap]=buildcmap('wygbr');
% %try the output cmap:
% im=imread('cameraman.tif');
% imshow(im), colorbar
% colormap(cmap) %will use the output colormap
%
% First version: 14 Feb. 2013
% sohrabinia.m@gmail.com
%--------------------------------------------------------------------------

if nargin<1
    colors='myb';
end

if ~ischar(colors)
    error(['Error! colors must be a variable of type char with '...
        'color-names, such as ''r'', ''g'', etc., '...
        'type ''help buildcmap'' for more info']);
end

ncolors=length(colors)-1;


bins=round(255/ncolors);
% diff1=255-bins*ncolors;

vec=zeros(300,3);

switch colors(1)
    case 'w'
        vec(1,:)=1;
    case 'r'
        vec(1,:)=[226  26  26]/255;
    case 'g'
        vec(1,:)=[165 202  26]/255;
    case 'b'
        vec(1,:)=[0 166 214]/255;
    case 'c'
        vec(1,:)=[0 1 1];
    case 'm'
        vec(1,:)=[109  23 127]/255;
    case 'y'
        vec(1,:)=[255 196   0]/255;
    case 'k'
        vec(1,:)=[0 0 0];
    case 'o'
        vec(1,:)=[230  70  22]/255;
end


for i=1:ncolors
 beG=(i-1)*bins+1;
 enD=i*bins+1; %beG,enD
 switch colors(i+1)
     case 'w'
         vec(beG:enD,1)=linspace(vec(beG,1),1,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),1,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),1,bins+1)';%colors(i+1),beG,enD,
     case 'r'
         vec(beG:enD,1)=linspace(vec(beG,1),226/255,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),26/255,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),26/255,bins+1)';%colors(i+1),beG,enD
     case 'g'
         vec(beG:enD,1)=linspace(vec(beG,1),165/255,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),202/255,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),26/255,bins+1)';%colors(i+1),beG,enD
     case 'b'        
         vec(beG:enD,1)=linspace(vec(beG,1),0,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),166/255,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),214/255,bins+1)';%colors(i+1),beG,enD
     case 'c'
         vec(beG:enD,1)=linspace(vec(beG,1),0,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),1,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),1,bins+1)';%colors(i+1),beG,enD
     case 'm'
         vec(beG:enD,1)=linspace(vec(beG,1),109/255,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),23/255,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),127/255,bins+1)';
     case 'y'
         vec(beG:enD,1)=linspace(vec(beG,1),1,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),196/255,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),0,bins+1)';
     case 'k'
         vec(beG:enD,1)=linspace(vec(beG,1),0,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),0,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),0,bins+1)';
     case 'o'
         vec(beG:enD,1)=linspace(vec(beG,1),230/255,bins+1)';
         vec(beG:enD,2)=linspace(vec(beG,2),70/255,bins+1)';
         vec(beG:enD,3)=linspace(vec(beG,3),22/255,bins+1)';
 end
end
cmap=vec(1:bins*ncolors,:);
end %end of buildcmap
